"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.refined = exports.nominal = exports.isRefined = exports.isNominal = exports.errors = exports.error = exports.all = exports.RefinedConstructorsTypeId = exports.NominalConstructorTypeId = exports.BrandTypeId = void 0;
var Either = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@fp-ts/core/Either"));
var _Function = /*#__PURE__*/require("@fp-ts/core/Function");
var Option = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@fp-ts/core/Option"));
var ReadonlyArray = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@fp-ts/core/ReadonlyArray"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * @since 1.0.0
 */

/**
 * @since 1.0.0
 * @category symbols
 */
const BrandTypeId = /*#__PURE__*/Symbol.for("@fp-ts/data/Brand");
/**
 * @since 1.0.0
 * @category symbols
 */
exports.BrandTypeId = BrandTypeId;
const NominalConstructorTypeId = /*#__PURE__*/Symbol.for("@fp-ts/data/Brand/Nominal");
/**
 * @since 1.0.0
 * @category symbols
 */
exports.NominalConstructorTypeId = NominalConstructorTypeId;
const RefinedConstructorsTypeId = /*#__PURE__*/Symbol.for("@fp-ts/data/Brand/Refined");
/**
 * @since 1.0.0
 * @category constructors
 */
exports.RefinedConstructorsTypeId = RefinedConstructorsTypeId;
const error = (message, meta) => [{
  message,
  meta
}];
/**
 * @since 1.0.0
 * @category constructors
 */
exports.error = error;
const errors = (...errors) => ReadonlyArray.flatten(errors);
/**
 * Returns `true` if the provided `Brand` is nominal, `false` otherwise.
 *
 * @since 1.0.0
 * @category refinements
 */
exports.errors = errors;
const isNominal = u => NominalConstructorTypeId in u;
/**
 * Returns `true` if the provided `Brand` is refined, `false` otherwise.
 *
 * @since 1.0.0
 * @category refinements
 */
exports.isNominal = isNominal;
const isRefined = u => RefinedConstructorsTypeId in u;
/**
 * @since 1.0.0
 * @category constructors
 */
exports.isRefined = isRefined;
const nominal = () =>
// @ts-expect-error
Object.assign(args => args, {
  [NominalConstructorTypeId]: NominalConstructorTypeId
});
/**
 * @since 1.0.0
 * @category constructors
 */
exports.nominal = nominal;
const refined = (refinement, onFailure) => {
  const either = args => refinement(args) ? Either.right(args) : Either.left(onFailure(args));
  return {
    [RefinedConstructorsTypeId]: RefinedConstructorsTypeId,
    of: args => (0, _Function.pipe)(either(args), Either.getOrThrow(_Function.identity)),
    option: args => Option.fromEither(either(args)),
    either,
    refine: args => Either.isRight(either(args))
  };
};
/**
 * Composes two brands together to form a single branded type.
 *
 * @since 1.0.0
 * @category mutations
 */
exports.refined = refined;
const all = (...brands) => {
  const either = args => {
    let result = Either.right(args);
    for (const brand of brands) {
      if (isRefined(brand)) {
        const nextResult = brand.either(args);
        if (Either.isLeft(result) && Either.isLeft(nextResult)) {
          result = Either.left([...result.left, ...nextResult.left]);
        } else {
          result = Either.isLeft(result) ? result : nextResult;
        }
      }
    }
    return result;
  };
  return {
    [RefinedConstructorsTypeId]: RefinedConstructorsTypeId,
    of: args => (0, _Function.pipe)(either(args), Either.getOrThrow(_Function.identity)),
    option: args => Option.fromEither(either(args)),
    either,
    refine: args => Either.isRight(either(args))
  };
};
exports.all = all;
//# sourceMappingURL=Brand.js.map