/**
 * @since 1.0.0
 */
import type { Option } from "@fp-ts/core/Option";
import type { Equal } from "@fp-ts/data/Equal";
declare const TagTypeId: unique symbol;
/**
 * @since 1.0.0
 * @category symbol
 */
export type TagTypeId = typeof TagTypeId;
/**
 * @since 1.0.0
 * @category models
 */
export interface Tag<Service> {
    readonly _id: TagTypeId;
    readonly _S: (_: Service) => Service;
}
/**
 * @since 1.0.0
 */
export declare namespace Tag {
    type Service<T extends Tag<any>> = T extends Tag<infer A> ? A : never;
}
/**
 * Specifying the key will make the Tag global, meaning two tags with the same
 * key will map to the same instance.
 *
 * Note: this is useful for cases where live reload can happen and it is
 * desireable to preserve the instance across reloads.
 *
 * @since 1.0.0
 * @category constructors
 */
export declare const Tag: <Service>(key?: string) => Tag<Service>;
declare const TypeId: unique symbol;
/**
 * @since 1.0.0
 * @category symbol
 */
export type TypeId = typeof TypeId;
/**
 * @since 1.0.0
 * @category models
 */
export type Tags<R> = R extends infer S ? Tag<S> : never;
/**
 * @since 1.0.0
 * @category models
 */
export interface Context<Services> extends Equal {
    readonly _id: TypeId;
    readonly _S: (_: Services) => unknown;
}
/**
 * @since 1.0.0
 * @category guards
 */
export declare const isContext: (u: unknown) => u is Context<never>;
/**
 * @since 1.0.0
 * @category guards
 */
export declare const isTag: (u: unknown) => u is Tag<never>;
/**
 * @since 1.0.0
 * @category constructors
 */
export declare const empty: () => Context<never>;
/**
 * @since 1.0.0
 * @category constructors
 */
export declare const make: <T extends Tag<any>>(tag: T, service: Tag.Service<T>) => Context<Tag.Service<T>>;
/**
 * @since 1.0.0
 * @category mutations
 */
export declare const add: {
    <Services, T extends Tag<any>>(self: Context<Services>, tag: Tag<T>, service: Tag.Service<T>): Context<Services | Tag.Service<T>>;
    <T extends Tag<any>>(tag: T, service: Tag.Service<T>): <Services>(self: Context<Services>) => Context<Tag.Service<T> | Services>;
};
/**
 * @since 1.0.0
 * @category getters
 */
export declare const get: {
    <Services, T extends Tags<Services>>(self: Context<Services>, tag: T): T extends Tag<infer S> ? S : never;
    <Services, T extends Tags<Services>>(tag: T): (self: Context<Services>) => T extends Tag<infer S> ? S : never;
};
/**
 * @since 1.0.0
 * @category unsafe
 */
export declare const unsafeGet: {
    <Services, S>(self: Context<Services>, tag: Tag<S>): S;
    <S>(tag: Tag<S>): <Services>(self: Context<Services>) => S;
};
/**
 * @since 1.0.0
 * @category getters
 */
export declare const getOption: {
    <Services, S>(self: Context<Services>, tag: Tag<S>): Option<S>;
    <S>(tag: Tag<S>): <Services>(self: Context<Services>) => Option<S>;
};
/**
 * @since 1.0.0
 * @category mutations
 */
export declare const merge: {
    <Services, R1>(self: Context<Services>, that: Context<R1>): Context<Services | R1>;
    <R1>(that: Context<R1>): <Services>(self: Context<Services>) => Context<R1 | Services>;
};
/**
 * @since 1.0.0
 * @category mutations
 */
export declare const prune: <Services, S extends Array<Tags<Services>>>(...tags: S) => (self: Context<Services>) => Context<{
    [k in keyof S]: Tag.Service<S[k]>;
}[number]>;
export {};
//# sourceMappingURL=Context.d.ts.map