declare const TypeId: unique symbol;
/**
 * @since 1.0.0
 * @category symbol
 */
export type TypeId = typeof TypeId;
/**
 * @since 1.0.0
 * @category models
 */
export interface MutableRef<T> {
    readonly _id: TypeId;
    readonly _T: (_: never) => T;
    /**
     * @since 1.0.0
     * @category general
     */
    get<T>(this: MutableRef<T>): T;
    /**
     * @since 1.0.0
     * @category general
     */
    set<T>(this: MutableRef<T>, value: T): MutableRef<T>;
    /**
     * @since 1.0.0
     * @category general
     */
    update<T>(this: MutableRef<T>, f: (value: T) => T): MutableRef<T>;
    /**
     * @since 1.0.0
     * @category general
     */
    updateAndGet<T>(this: MutableRef<T>, f: (value: T) => T): T;
    /**
     * @since 1.0.0
     * @category general
     */
    getAndUpdate<T>(this: MutableRef<T>, f: (value: T) => T): T;
    /**
     * @since 1.0.0
     * @category general
     */
    setAndGet<T>(this: MutableRef<T>, value: T): T;
    /**
     * @since 1.0.0
     * @category general
     */
    getAndSet<T>(this: MutableRef<T>, value: T): T;
    /**
     * @since 1.0.0
     * @category general
     */
    compareAndSet<T>(this: MutableRef<T>, oldValue: T, newValue: T): boolean;
    /**
     * @since 1.0.0
     * @category general
     */
    pipe<T, B>(this: MutableRef<T>, f: (_: MutableRef<T>) => B): B;
}
/**
 * @since 1.0.0
 * @category constructors
 */
export declare const make: <T>(value: T) => MutableRef<T>;
/**
 * @since 1.0.0
 * @category general
 */
export declare const compareAndSet: {
    <T>(self: MutableRef<T>, oldValue: T, newValue: T): boolean;
    <T>(oldValue: T, newValue: T): (self: MutableRef<T>) => boolean;
};
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const decrement: (self: MutableRef<number>) => MutableRef<number>;
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const decrementAndGet: (self: MutableRef<number>) => number;
/**
 * @since 1.0.0
 * @category general
 */
export declare const get: <T>(self: MutableRef<T>) => T;
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const getAndDecrement: (self: MutableRef<number>) => number;
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const getAndIncrement: (self: MutableRef<number>) => number;
/**
 * @since 1.0.0
 * @category general
 */
export declare const getAndSet: {
    <T>(self: MutableRef<T>, value: T): T;
    <T>(value: T): (self: MutableRef<T>) => T;
};
/**
 * @since 1.0.0
 * @category general
 */
export declare const getAndUpdate: {
    <T>(self: MutableRef<T>, f: (value: T) => T): T;
    <T>(f: (value: T) => T): (self: MutableRef<T>) => T;
};
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const increment: (self: MutableRef<number>) => MutableRef<number>;
/**
 * @since 1.0.0
 * @category numeric
 */
export declare const incrementAndGet: (self: MutableRef<number>) => number;
/**
 * @since 1.0.0
 * @category general
 */
export declare const set: {
    <T>(self: MutableRef<T>, value: T): MutableRef<T>;
    <T>(value: T): (self: MutableRef<T>) => MutableRef<T>;
};
/**
 * @since 1.0.0
 * @category general
 */
export declare const setAndGet: {
    <T>(self: MutableRef<T>, value: T): T;
    <T>(value: T): (self: MutableRef<T>) => T;
};
/**
 * @since 1.0.0
 * @category general
 */
export declare const update: {
    <T>(self: MutableRef<T>, f: (value: T) => T): MutableRef<T>;
    <T>(f: (value: T) => T): (self: MutableRef<T>) => MutableRef<T>;
};
/**
 * @since 1.0.0
 * @category general
 */
export declare const updateAndGet: {
    <T>(self: MutableRef<T>, f: (value: T) => T): T;
    <T>(f: (value: T) => T): (self: MutableRef<T>) => T;
};
/**
 * @since 1.0.0
 * @category boolean
 */
export declare const toggle: (self: MutableRef<boolean>) => MutableRef<boolean>;
export {};
//# sourceMappingURL=MutableRef.d.ts.map