/**
 * @since 1.0.0
 */
import * as Dual from "@fp-ts/data/Dual";
import * as Equal from "@fp-ts/data/Equal";
import * as Hash from "@fp-ts/data/Hash";
const TypeId = /*#__PURE__*/Symbol.for("@fp-ts/data/Duration");
/** @internal */
class DurationImpl {
  constructor(millis) {
    this.millis = millis;
    this._id = TypeId;
  }
  [Hash.symbol]() {
    return Hash.hash(this.millis);
  }
  [Equal.symbol](that) {
    return isDuration(that) && this.millis === that.millis;
  }
}
/**
 * @since 1.0.0
 * @category guards
 */
export const isDuration = u => typeof u === "object" && u != null && "_id" in u && u["_id"] === TypeId;
/**
 * @since 1.0.0
 * @category constructors
 */
export const zero = /*#__PURE__*/new DurationImpl(0);
/**
 * @since 1.0.0
 * @category constructors
 */
export const infinity = /*#__PURE__*/new DurationImpl(Infinity);
/**
 * @since 1.0.0
 * @category constructors
 */
export const millis = millis => new DurationImpl(millis);
/**
 * @since 1.0.0
 * @category constructors
 */
export const seconds = seconds => new DurationImpl(seconds * 1000);
/**
 * @since 1.0.0
 * @category constructors
 */
export const minutes = minutes => new DurationImpl(minutes * 60000);
/**
 * @since 1.0.0
 * @category constructors
 */
export const hours = hours => new DurationImpl(hours * 3600000);
/**
 * @since 1.0.0
 * @category constructors
 */
export const days = days => new DurationImpl(days * 86400000);
/**
 * @since 1.0.0
 * @category constructors
 */
export const weeks = weeks => new DurationImpl(weeks * 604800000);
/**
 * @since 1.0.0
 * @category mutations
 */
export const times = /*#__PURE__*/Dual.dual(2, (self, times) => new DurationImpl(self.millis * times));
/**
 * @since 1.0.0
 * @category mutations
 */
export const add = /*#__PURE__*/Dual.dual(2, (self, that) => new DurationImpl(self.millis + that.millis));
/**
 * @since 1.0.0
 * @category mutations
 */
export const subtract = /*#__PURE__*/Dual.dual(2, (self, that) => new DurationImpl(self.millis - that.millis));
/**
 * @since 1.0.0
 * @category comparisons
 */
export const lessThan = /*#__PURE__*/Dual.dual(2, (self, that) => self.millis < that.millis);
/**
 * @since 1.0.0
 * @category comparisons
 */
export const lessThanOrEqualTo = /*#__PURE__*/Dual.dual(2, (self, that) => self.millis <= that.millis);
/**
 * @since 1.0.0
 * @category comparisons
 */
export const greaterThan = /*#__PURE__*/Dual.dual(2, (self, that) => self.millis > that.millis);
/**
 * @since 1.0.0
 * @category comparisons
 */
export const greaterThanOrEqualTo = /*#__PURE__*/Dual.dual(2, (self, that) => self.millis >= that.millis);
/**
 * @since 1.0.0
 * @category comparisons
 */
export const equals = /*#__PURE__*/Dual.dual(2, (self, that) => self.millis === that.millis);
//# sourceMappingURL=Duration.mjs.map