var _a;
/**
 * @since 1.0.0
 */
import * as E from "@fp-ts/core/Either";
import { identity, pipe } from "@fp-ts/core/Function";
import * as O from "@fp-ts/core/Option";
import * as T from "@fp-ts/core/These";
/**
 * @category symbols
 * @since 1.0.0
 */
export const GenKindTypeId = /*#__PURE__*/Symbol.for("@fp-ts/data/Gen/GenKind");
/**
 * @category constructors
 * @since 1.0.0
 */
export class GenKindImpl {
  constructor(
  /**
   * @since 1.0.0
   */
  value) {
    this.value = value;
    /**
     * @since 1.0.0
     */
    this[_a] = GenKindTypeId;
  }
  /**
   * @since 1.0.0
   */
  get _F() {
    return identity;
  }
  /**
   * @since 1.0.0
   */
  get _R() {
    return _ => _;
  }
  /**
   * @since 1.0.0
   */
  get _O() {
    return _ => _;
  }
  /**
   * @since 1.0.0
   */
  get _E() {
    return _ => _;
  }
  /**
   * @since 1.0.0
   */
  [(_a = GenKindTypeId, Symbol.iterator)]() {
    return new SingleShotGen(this);
  }
}
/**
 * @category constructors
 * @since 1.0.0
 */
export class SingleShotGen {
  constructor(self) {
    this.self = self;
    this.called = false;
  }
  /**
   * @since 1.0.0
   */
  next(a) {
    return this.called ? {
      value: a,
      done: true
    } : (this.called = true, {
      value: this.self,
      done: false
    });
  }
  /**
   * @since 1.0.0
   */
  return(a) {
    return {
      value: a,
      done: true
    };
  }
  /**
   * @since 1.0.0
   */
  throw(e) {
    throw e;
  }
  /**
   * @since 1.0.0
   */
  [Symbol.iterator]() {
    return new SingleShotGen(this.self);
  }
}
/**
 * @category constructors
 * @since 1.0.0
 */
export const makeGenKind = kind => new GenKindImpl(kind);
/**
 * @category adapters
 * @since 1.0.0
 */
export const adapter = () => kind => new GenKindImpl(kind);
function runGen(F, state, iterator) {
  if (state.done) {
    return F.of(state.value);
  }
  return F.flatMap(val => {
    const next = iterator.next(val);
    return runGen(F, next, iterator);
  })(state.value.value);
}
/**
 * @category constructors
 * @since 1.0.0
 */
export const singleShot = F => adapter => body => pipe(F.of(void 0), F.flatMap(() => {
  const iterator = body(adapter);
  const state = iterator.next();
  // @ts-expect-error
  return runGen(F, state, iterator);
}));
/**
 * @since 1.0.0
 * @category instances
 */
export const Either = /*#__PURE__*/singleShot(E.Monad)( /*#__PURE__*/adapter());
/**
 * @since 1.0.0
 * @category instances
 */
export const These = /*#__PURE__*/singleShot(T.Monad)( /*#__PURE__*/adapter());
/**
 * @since 1.0.0
 * @category instances
 */
export const Option = /*#__PURE__*/singleShot(O.Monad)( /*#__PURE__*/adapter());
//# sourceMappingURL=Gen.mjs.map